@@ -22,11 +22,16 @@
 #define __EMPATHY_ACCOUNT_WIDGET_IRC_H__
 
 #include <gtk/gtk.h>
-#include <libmissioncontrol/mc-account.h>
+#include <libempathy-gtk/empathy-account-widget.h>
 
 G_BEGIN_DECLS
 
-GtkWidget * empathy_account_widget_irc_new (McAccount *account);
+void empathy_account_widget_irc_build (EmpathyAccountWidget *self,
+    const char *filename,
+    GtkWidget **table_common_settings);
+
+void empathy_account_widget_irc_build_simple (EmpathyAccountWidget *self,
+    const char *filename);
 
 G_END_DECLS
 
