@@ -85,7 +85,11 @@ static void empathy_contact_set_location (EmpathyContact *contact,
 static void set_capabilities_from_tp_caps (EmpathyContact *self,
     TpCapabilities *caps);
 
+static void contact_set_avatar (EmpathyContact *contact,
+    EmpathyAvatar *avatar);
 static void contact_set_avatar_from_tp_contact (EmpathyContact *contact);
+static gboolean contact_load_avatar_cache (EmpathyContact *contact,
+    const gchar *token);
 
 G_DEFINE_TYPE (EmpathyContact, empathy_contact, G_TYPE_OBJECT);
 
@@ -266,7 +270,7 @@ empathy_contact_class_init (EmpathyContactClass *class)
         "Avatar image",
         "The avatar image",
         EMPATHY_TYPE_AVATAR,
-        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (object_class,
       PROP_PRESENCE,
@@ -478,9 +482,6 @@ contact_set_property (GObject *object,
       case PROP_ALIAS:
         empathy_contact_set_alias (contact, g_value_get_string (value));
         break;
-      case PROP_AVATAR:
-        empathy_contact_set_avatar (contact, g_value_get_boxed (value));
-        break;
       case PROP_PRESENCE:
         empathy_contact_set_presence (contact, g_value_get_uint (value));
         break;
@@ -531,29 +532,12 @@ empathy_contact_from_tpl_contact (TpAccount *account,
       NULL);
 
   if (!EMP_STR_EMPTY (tpl_entity_get_avatar_token (tpl_entity)))
-    empathy_contact_load_avatar_cache (retval,
+    contact_load_avatar_cache (retval,
         tpl_entity_get_avatar_token (tpl_entity));
 
   return retval;
 }
 
-EmpathyContact *
-empathy_contact_new_for_log (TpAccount *account,
-                             const gchar *id,
-                             const gchar *alias,
-                             gboolean is_user)
-{
-  g_return_val_if_fail (id != NULL, NULL);
-  g_assert (account != NULL);
-
-  return g_object_new (EMPATHY_TYPE_CONTACT,
-      "account", account,
-      "id", id,
-      "alias", alias,
-      "is-user", is_user,
-      NULL);
-}
-
 TpContact *
 empathy_contact_get_tp_contact (EmpathyContact *contact)
 {
@@ -724,9 +708,9 @@ empathy_contact_get_avatar (EmpathyContact *contact)
   return priv->avatar;
 }
 
-void
-empathy_contact_set_avatar (EmpathyContact *contact,
-                            EmpathyAvatar *avatar)
+static void
+contact_set_avatar (EmpathyContact *contact,
+                    EmpathyAvatar *avatar)
 {
   EmpathyContactPriv *priv;
 
@@ -838,7 +822,11 @@ empathy_contact_set_persona (EmpathyContact *contact,
     return;
 
   if (priv->persona != NULL)
-    g_object_unref (priv->persona);
+    {
+      g_signal_handlers_disconnect_by_func (priv->persona,
+          folks_persona_notify_cb, contact);
+      g_object_unref (priv->persona);
+    }
   priv->persona = g_object_ref (persona);
 
   g_signal_connect (priv->persona, "notify",
@@ -1048,8 +1036,10 @@ empathy_contact_is_online (EmpathyContact *contact)
       case TP_CONNECTION_PRESENCE_TYPE_OFFLINE:
       case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
       case TP_CONNECTION_PRESENCE_TYPE_ERROR:
-      case TP_CONNECTION_PRESENCE_TYPE_UNSET:
         return FALSE;
+      /* Contacts without presence are considered online so we can display IRC
+       * contacts in rooms. */
+      case TP_CONNECTION_PRESENCE_TYPE_UNSET:
       case TP_CONNECTION_PRESENCE_TYPE_AVAILABLE:
       case TP_CONNECTION_PRESENCE_TYPE_AWAY:
       case TP_CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY:
@@ -1167,9 +1157,9 @@ contact_get_avatar_filename (EmpathyContact *contact,
   return avatar_file;
 }
 
-gboolean
-empathy_contact_load_avatar_cache (EmpathyContact *contact,
-                                   const gchar *token)
+static gboolean
+contact_load_avatar_cache (EmpathyContact *contact,
+                           const gchar *token)
 {
   EmpathyAvatar *avatar = NULL;
   gchar *filename;
@@ -1195,9 +1185,8 @@ empathy_contact_load_avatar_cache (EmpathyContact *contact,
   if (data)
     {
       DEBUG ("Avatar loaded from %s", filename);
-      avatar = empathy_avatar_new ((guchar *) data, len, NULL, g_strdup (token),
-          filename);
-      empathy_contact_set_avatar (contact, avatar);
+      avatar = empathy_avatar_new ((guchar *) data, len, NULL, filename);
+      contact_set_avatar (contact, avatar);
       empathy_avatar_unref (avatar);
     }
   else
@@ -1228,11 +1217,10 @@ empathy_avatar_get_type (void)
  * @data: the avatar data
  * @len: the size of avatar data
  * @format: the mime type of the avatar image
- * @token: the token of the avatar
  * @filename: the filename where the avatar is stored in cache
  *
  * Create a #EmpathyAvatar from the provided data. This function takes the
- * ownership of @data, @format, @token and @filename.
+ * ownership of @data, @format and @filename.
  *
  * Returns: a new #EmpathyAvatar
  */
@@ -1240,7 +1228,6 @@ EmpathyAvatar *
 empathy_avatar_new (guchar *data,
                     gsize len,
                     gchar *format,
-                    gchar *token,
                     gchar *filename)
 {
   EmpathyAvatar *avatar;
@@ -1249,7 +1236,6 @@ empathy_avatar_new (guchar *data,
   avatar->data = data;
   avatar->len = len;
   avatar->format = format;
-  avatar->token = token;
   avatar->filename = filename;
   avatar->refcount = 1;
 
@@ -1266,7 +1252,6 @@ empathy_avatar_unref (EmpathyAvatar *avatar)
     {
       g_free (avatar->data);
       g_free (avatar->format);
-      g_free (avatar->token);
       g_free (avatar->filename);
       g_slice_free (EmpathyAvatar, avatar);
     }
@@ -1650,10 +1635,8 @@ contact_set_avatar_from_tp_contact (EmpathyContact *contact)
 {
   EmpathyContactPriv *priv = GET_PRIV (contact);
   const gchar *mime;
-  const gchar *token;
   GFile *file;
 
-  token = tp_contact_get_avatar_token (priv->tp_contact);
   mime = tp_contact_get_avatar_mime_type (priv->tp_contact);
   file = tp_contact_get_avatar_file (priv->tp_contact);
 
@@ -1664,14 +1647,14 @@ contact_set_avatar_from_tp_contact (EmpathyContact *contact)
       gsize len;
 
       g_file_load_contents (file, NULL, &data, &len, NULL, NULL);
-      avatar = empathy_avatar_new ((guchar *) data, len, g_strdup (mime), g_strdup (token),
+      avatar = empathy_avatar_new ((guchar *) data, len, g_strdup (mime),
           g_file_get_path (file));
-      empathy_contact_set_avatar (contact, avatar);
+      contact_set_avatar (contact, avatar);
       empathy_avatar_unref (avatar);
     }
   else
     {
-      empathy_contact_set_avatar (contact, NULL);
+      contact_set_avatar (contact, NULL);
     }
 }
 
